@@ -21,23 +21,21 @@ import cvxpy.interface as intf
 import cvxpy.settings as s
 from cvxpy.error import SolverError
 from cvxpy.reductions.solution import Solution, failure_solution
+from cvxpy.reductions.solvers import utilities
+from cvxpy.reductions.solvers.conic_solvers.highs_conif import (  # importing to avoid duplication
+    set_column_names_from_variables,
+    unpack_highs_options_inplace,
+)
 from cvxpy.reductions.solvers.qp_solvers.qp_solver import QpSolver
 from cvxpy.utilities.citations import CITATION_DICT
 
 
-def unpack_highs_options_inplace(solver_opts) -> None:
-    # Users can pass options inside a nested dict -- e.g. to circumvent a name clash
-    highs_options = solver_opts.pop("highs_options", dict())
-
-    # merge via update(dict(...)) is needed to avoid silently over-writing options
-    solver_opts.update(dict(**solver_opts, **highs_options))
-
-
 class HIGHS(QpSolver):
     """QP interface for the HiGHS solver"""
 
     # Note that HiGHS does not support MIQP but supports MILP
     MIP_CAPABLE = False
+    BOUNDED_VARIABLES = True
 
     # Map of HiGHS status to CVXPY status.
     STATUS_MAP = {
@@ -94,7 +92,21 @@ class HIGHS(QpSolver):
             # add duals if not a MIP.
             dual_vars = None
             if not inverse_data[HIGHS.IS_MIP]:
-                dual_vars = {HIGHS.DUAL_VAR_ID: -np.array(results["solution"].row_dual)}
+                # Build dual vars dict keyed by constraint IDs
+                # HiGHS returns duals for [eq_constrs; ineq_constrs]
+                y = -np.array(results["solution"].row_dual)
+                n_eq = inverse_data[self.DIMS].zero
+                eq_dual = utilities.get_dual_values(
+                    y[:n_eq],
+                    utilities.extract_dual_value,
+                    inverse_data[self.EQ_CONSTR])
+                ineq_dual = utilities.get_dual_values(
+                    y[n_eq:],
+                    utilities.extract_dual_value,
+                    inverse_data[self.NEQ_CONSTR])
+                dual_vars = {}
+                dual_vars.update(eq_dual)
+                dual_vars.update(ineq_dual)
             attr[s.NUM_ITERS] = (
                 results["info"].ipm_iteration_count
                 + results["info"].crossover_iteration_count
@@ -171,8 +183,10 @@ class HIGHS(QpSolver):
         lp.a_matrix_.value_ = A.data
 
         # Define Variable bounds
-        lp.col_lower_ = -inf * np.ones(shape=lp.num_col_, dtype=q.dtype)
-        lp.col_upper_ = inf * np.ones(shape=lp.num_col_, dtype=q.dtype)
+        lb = data[s.LOWER_BOUNDS]
+        ub = data[s.UPPER_BOUNDS]
+        lp.col_lower_ = np.full(lp.num_col_, -inf, dtype=q.dtype) if lb is None else lb.copy()
+        lp.col_upper_ = np.full(lp.num_col_, inf, dtype=q.dtype) if ub is None else ub.copy()
 
         # note that we count actual nonzeros because
         # parameter values could make the problem linear
@@ -190,6 +204,7 @@ class HIGHS(QpSolver):
 
         # setup options
         unpack_highs_options_inplace(solver_opts)
+        write_model_file = solver_opts.pop("write_model_file", None)
         solver.setOptionValue("log_to_console", verbose)
         for name, value in solver_opts.items():
             # note that calling setOptionValue directly on the solver
@@ -200,9 +215,16 @@ class HIGHS(QpSolver):
                     f"HIGHS returned status kError for option (name, value): ({name}, {value})"
                 )
 
+        if write_model_file:
+            # TODO: Names can be collected upstream more systematically
+            # (or in the parent class) to be used by all solvers.
+            set_column_names_from_variables(lp, data[s.PARAM_PROB].variables)
 
         solver.passModel(model)
 
+        if write_model_file:
+            solver.writeModel(write_model_file)
+
         if warm_start and solver_cache is not None and self.name() in solver_cache:
             old_solver, old_data, old_result = solver_cache[self.name()]
             old_status = self.STATUS_MAP.get(old_result["model_status"], s.SOLVER_ERROR)
