@@ -340,6 +340,98 @@ class TestBackward(BaseTest):
         self.assertIn(0.0, A.data)
 
 
+class TestBackwardComplex(BaseTest):
+    """Test backward/forward differentiation with complex parameters."""
+    def setUp(self) -> None:
+        try:
+            import diffcp
+            diffcp  # for flake8
+        except ModuleNotFoundError:
+            self.skipTest("diffcp not installed.")
+
+    def test_backward_real_and_imag(self) -> None:
+        """Backward differentiation through real and imag parts of complex parameter."""
+        p = cp.Parameter(complex=True)
+        x = cp.Variable()
+        y = cp.Variable()
+        # minimize (x - real(p))^2 + (y - imag(p))^2
+        prob = cp.Problem(
+            cp.Minimize(cp.square(x - cp.real(p)) + cp.square(y - cp.imag(p)))
+        )
+
+        p.value = np.array(3.0 + 4.0j)
+        prob.solve(requires_grad=True)
+        self.assertAlmostEqual(x.value, 3.0, places=3)
+        self.assertAlmostEqual(y.value, 4.0, places=3)
+
+        x.gradient = 1.0
+        y.gradient = 1.0
+        prob.backward()
+
+        # Gradient follows PyTorch convention: grad = d/d(real) + j*d/d(imag)
+        # dx*/d(real(p)) = 1, dy*/d(imag(p)) = 1 => p.gradient = 1 + 1j
+        self.assertAlmostEqual(np.real(p.gradient), 1.0, places=3)
+        self.assertAlmostEqual(np.imag(p.gradient), 1.0, places=3)
+
+    def test_forward_complex_delta(self) -> None:
+        """Forward differentiation with complex delta."""
+        p = cp.Parameter(complex=True)
+        x = cp.Variable()
+        y = cp.Variable()
+        prob = cp.Problem(cp.Minimize(x + y), [x >= cp.real(p), y >= cp.imag(p)])
+
+        p.value = np.array(3.0 + 4.0j)
+        prob.solve(requires_grad=True)
+        self.assertAlmostEqual(x.value, 3.0, places=3)
+        self.assertAlmostEqual(y.value, 4.0, places=3)
+
+        # Perturb both parts
+        p.delta = 2.0 + 3.0j
+        prob.derivative()
+
+        # x* = real(p), y* = imag(p)
+        self.assertAlmostEqual(x.delta, 2.0, places=3)
+        self.assertAlmostEqual(y.delta, 3.0, places=3)
+
+    def test_backward_hermitian_param(self) -> None:
+        """Backward differentiation with Hermitian parameter."""
+        n = 2
+        P = cp.Parameter((n, n), hermitian=True)
+        x = cp.Variable()
+        # Minimize x such that x*I >= P
+        prob = cp.Problem(cp.Minimize(x), [x * np.eye(n) >> P])
+
+        P_val = np.array([[1.0, 0.5j], [-0.5j, 2.0]])
+        P.value = P_val
+        prob.solve(requires_grad=True)
+        max_eig = np.max(np.linalg.eigvalsh(P_val))
+        self.assertAlmostEqual(x.value, max_eig, places=3)
+
+        x.gradient = 1.0
+        prob.backward()
+
+        # Gradient should be Hermitian
+        self.assertTrue(np.allclose(P.gradient, P.gradient.conj().T))
+
+    def test_forward_hermitian_delta(self) -> None:
+        """Forward differentiation with Hermitian parameter delta."""
+        n = 2
+        P = cp.Parameter((n, n), hermitian=True)
+        x = cp.Variable()
+        prob = cp.Problem(cp.Minimize(x), [x * np.eye(n) >> P])
+
+        P_val = np.array([[1.0, 0.5j], [-0.5j, 2.0]])
+        P.value = P_val
+        prob.solve(requires_grad=True)
+
+        # Perturb with Hermitian delta
+        P.delta = np.array([[0.1, 0.05j], [-0.05j, 0.2]])
+        prob.derivative()
+
+        # x.delta should be scalar (perturbation of max eigenvalue)
+        self.assertTrue(np.isscalar(x.delta) or x.delta.size == 1)
+
+
 class TestBackwardDgp(BaseTest):
     """Test problem.backward() and problem.derivative()."""
     def setUp(self) -> None:
