@@ -35,13 +35,18 @@ from cvxpy.atoms import (
     sum_squares,
 )
 from cvxpy.expressions.variable import Variable
-from cvxpy.reductions.solvers.defines import INSTALLED_SOLVERS, QP_SOLVERS
+from cvxpy.reductions.solvers.defines import (
+    INSTALLED_CONIC_SOLVERS,
+    INSTALLED_SOLVERS,
+    QP_SOLVERS,
+    SOLVER_MAP_CONIC,
+)
 from cvxpy.tests.base_test import BaseTest
-from cvxpy.tests.solver_test_helpers import StandardTestLPs, StandardTestQPs
+from cvxpy.tests.solver_test_helpers import SolverTestHelper, StandardTestLPs, StandardTestQPs
 
 
-class TestQp(BaseTest):
-    """ Unit tests for the domain module. """
+class QPTestBase(BaseTest):
+    """Base class with shared test helpers for QP-style problems."""
 
     def setUp(self) -> None:
         self.a = Variable(name='a')
@@ -70,90 +75,77 @@ class TestQp(BaseTest):
         self.xsr = Variable(50, name='xsr')
         self.xef = Variable(80, name='xef')
 
-        # Check for all installed QP solvers
-        self.solvers = [x for x in QP_SOLVERS if x in INSTALLED_SOLVERS]
-
-        def is_mosek_available():
-            """Check if MOSEK is installed and a license is available."""
-            if 'MOSEK' not in INSTALLED_SOLVERS:
-                return False
-            try:
-                import mosek  # type: ignore
-                env = mosek.Env()
-                # Try to get license status (returns 0 if OK)
-                status = env.getlicense()
-                return status == mosek.rescode.ok
-            except Exception:
-                return False
-
-        def is_knitro_available():
-            """Check if KNITRO is installed and a license is available."""
-            if 'KNITRO' not in INSTALLED_SOLVERS:
-                return False
-            try:
-                import knitro  # type: ignore
-                # Try to create and delete a Knitro solver instance
-                kc = knitro.KN_new()
-                if kc is None:
-                    return False
-                knitro.KN_free(kc)
-                return True
-            except Exception:
-                return False
-
-        def is_xpress_available():
-            """Check if XPRESS is installed and a license is available."""
-            if 'XPRESS' not in INSTALLED_SOLVERS:
-                return False
-            try:
-                import xpress  # type: ignore
-                env = xpress.env()
-                status = env.getlicense()
-                return status == 0
-            except Exception:
-                return False
-        # Remove XPRESS if license is not available
-        if 'XPRESS' in self.solvers and not is_xpress_available():
-            self.solvers.remove('XPRESS')
-        if 'MOSEK' in self.solvers and not is_mosek_available():
-            self.solvers.remove('MOSEK')
-        if 'KNITRO' in self.solvers and not is_knitro_available():
-            self.solvers.remove('KNITRO')
-
     def solve_QP(self, problem, solver_name):
-        return problem.solve(solver=solver_name, verbose=False)
+        """Override in subclasses."""
+        raise NotImplementedError
 
-    def test_all_solvers(self) -> None:
-        for solver in self.solvers:
-            self.quad_over_lin(solver)
-            self.power(solver)
-            self.power_matrix(solver)
-            self.square_affine(solver)
-            self.quad_form(solver)
-            self.affine_problem(solver)
-            self.maximize_problem(solver)
-            self.abs(solver)
+    # License checking helpers - shared by subclasses
+    @staticmethod
+    def is_mosek_available():
+        """Check if MOSEK is installed and a license is available."""
+        if 'MOSEK' not in INSTALLED_SOLVERS:
+            return False
+        try:
+            import mosek  # type: ignore
+            env = mosek.Env()
+            status = env.getlicense()
+            return status == mosek.rescode.ok
+        except Exception:
+            return False
 
-            # Do we need the following functionality?
-            # self.norm_2(solver)
-            # self.mat_norm_2(solver)
+    @staticmethod
+    def is_knitro_available():
+        """Check if KNITRO is installed and a license is available."""
+        if 'KNITRO' not in INSTALLED_SOLVERS:
+            return False
+        try:
+            import knitro  # type: ignore
+            kc = knitro.KN_new()
+            if kc is None:
+                return False
+            knitro.KN_free(kc)
+            return True
+        except Exception:
+            return False
 
-            self.quad_form_coeff(solver)
-            self.quad_form_bound(solver)
-            self.regression_1(solver)
-            self.regression_2(solver)
-            self.rep_quad_form(solver)
+    @staticmethod
+    def is_xpress_available():
+        """Check if XPRESS is installed and a license is available."""
+        if 'XPRESS' not in INSTALLED_SOLVERS:
+            return False
+        try:
+            import xpress  # type: ignore
+            env = xpress.env()
+            status = env.getlicense()
+            return status == 0
+        except Exception:
+            return False
 
-            # slow tests:
-            self.control(solver)
-            self.sparse_system(solver)
-            self.smooth_ridge(solver)
-            self.huber_small(solver)
-            self.huber(solver)
-            self.equivalent_forms_1(solver)
-            self.equivalent_forms_2(solver)
-            self.equivalent_forms_3(solver)
+    def filter_licensed_solvers(self, solvers):
+        """Remove solvers that don't have valid licenses."""
+        result = list(solvers)
+        if 'XPRESS' in result and not self.is_xpress_available():
+            result.remove('XPRESS')
+        if 'MOSEK' in result and not self.is_mosek_available():
+            result.remove('MOSEK')
+        if 'KNITRO' in result and not self.is_knitro_available():
+            result.remove('KNITRO')
+        return result
 
+    def _check_kkt(self, problem, places=4):
+        """Verify KKT conditions for a solved problem."""
+        obj_pair = (problem.objective, None)
+        var_pairs = [(v, None) for v in problem.variables()]
+        con_pairs = [(c, None) for c in problem.constraints]
+        sth = SolverTestHelper(obj_pair, var_pairs, con_pairs)
+        # Problem already solved, just use the same problem object
+        sth.prob = problem
+        sth.check_primal_feasibility(places)
+        sth.check_complementarity(places)
+        sth.check_dual_domains(places)
+        sth.check_stationary_lagrangian(places)
+
+    # Test helper methods - shared by all subclasses
     def quad_over_lin(self, solver) -> None:
         p = Problem(Minimize(0.5 * quad_over_lin(abs(self.x-1), 1)),
                     [self.x <= -1])
@@ -164,6 +156,7 @@ class TestQp(BaseTest):
         for con in p.constraints:
             self.assertItemsAlmostEqual(np.array([2., 2.]),
                                         con.dual_value, places=4)
+        self._check_kkt(p, places=3)
 
     def abs(self, solver) -> None:
         u = Variable(2)
@@ -223,6 +216,7 @@ class TestQp(BaseTest):
             self.assertItemsAlmostEqual(z, var.value, places=4)
 
     def affine_problem(self, solver) -> None:
+        np.random.seed(0)
         A = np.random.randn(5, 2)
         A = np.maximum(A, 0)
         b = np.random.randn(5)
@@ -231,8 +225,10 @@ class TestQp(BaseTest):
         self.solve_QP(p, solver)
         for var in p.variables():
             self.assertItemsAlmostEqual([0., 0.], var.value, places=3)
+        self._check_kkt(p, places=3)
 
     def maximize_problem(self, solver) -> None:
+        np.random.seed(0)
         A = np.random.randn(5, 2)
         A = np.maximum(A, 0)
         b = np.random.randn(5)
@@ -241,6 +237,7 @@ class TestQp(BaseTest):
         self.solve_QP(p, solver)
         for var in p.variables():
             self.assertItemsAlmostEqual([0., 0.], var.value, places=3)
+        self._check_kkt(p, places=3)
 
     def norm_2(self, solver) -> None:
         A = np.random.randn(10, 5)
@@ -281,6 +278,7 @@ class TestQp(BaseTest):
         self.solve_QP(p, solver)
         for var in p.variables():
             self.assertItemsAlmostEqual(y_star, var.value, places=4)
+        self._check_kkt(p)
 
     def regression_1(self, solver) -> None:
         np.random.seed(1)
@@ -357,6 +355,10 @@ class TestQp(BaseTest):
         p = Problem(Minimize(.01 * sum_squares(self.force)), constraints)
         self.solve_QP(p, solver)
         self.assertAlmostEqual(1059.616, p.value, places=1)
+        # KKT check skipped: check_stationary_lagrangian fails for 2D matrix
+        # variables due to inconsistent gradient ordering (sum_squares uses
+        # C order, constraint terms use F order). TODO fix this
+        # self._check_kkt(p, places=3)
 
     def sparse_system(self, solver) -> None:
         m = 100
@@ -384,7 +386,7 @@ class TestQp(BaseTest):
         self.solve_QP(p, solver)
         self.assertAlmostEqual(0, p.value, places=4)
 
-    def huber_small(self, solver) -> None:
+    def huber_small(self, solver, places=4) -> None:
         # Solve the Huber regression problem
         x = Variable(3)
         objective = sum(huber(x))
@@ -392,17 +394,18 @@ class TestQp(BaseTest):
         # Solve problem with QP
         p = Problem(Minimize(objective), [x[2] >= 3])
         self.solve_QP(p, solver)
-        self.assertAlmostEqual(3, x.value[2], places=4)
-        self.assertAlmostEqual(5, objective.value, places=4)
+        self.assertAlmostEqual(3, x.value[2], places=places)
+        self.assertAlmostEqual(5, objective.value, places=places)
+        self._check_kkt(p, places=places)
 
     def huber(self, solver) -> None:
         # Generate problem data
         n = 3
         m = 5
-        data = [0.89, 0.39, 0.96, 0.34, 0.68, 0.18, 0.63 ,0.42, 0.51, 0.66, 0.43, 0.77]
+        data = [0.89, 0.39, 0.96, 0.34, 0.68, 0.18, 0.63, 0.42, 0.51, 0.66, 0.43, 0.77]
         indices = [0, 1, 2, 3, 4, 2, 3, 0, 1, 2, 3, 4]
         indptr = [0, 5, 7, 12]
-        A = sp.csc_array((data, indices, indptr), shape=(m,n))
+        A = sp.csc_array((data, indices, indptr), shape=(m, n))
         x_true = np.random.randn(n) / np.sqrt(n)
         ind95 = (np.random.rand(m) < 0.95).astype(float)
         b = A.dot(x_true) + np.multiply(0.5*np.random.randn(m), ind95) \
@@ -436,6 +439,7 @@ class TestQp(BaseTest):
         p1 = Problem(Minimize(obj1), cons)
         self.solve_QP(p1, solver)
         self.assertAlmostEqual(p1.value, 68.1119420108, places=4)
+        self._check_kkt(p1, places=4)
 
     def equivalent_forms_2(self, solver) -> None:
         m = 100
@@ -458,6 +462,7 @@ class TestQp(BaseTest):
         p2 = Problem(Minimize(obj2), cons)
         self.solve_QP(p2, solver)
         self.assertAlmostEqual(p2.value, 68.1119420108, places=4)
+        self._check_kkt(p2, places=4)
 
     def equivalent_forms_3(self, solver) -> None:
         m = 100
@@ -480,7 +485,55 @@ class TestQp(BaseTest):
 
         p3 = Problem(Minimize(obj3), cons)
         self.solve_QP(p3, solver)
+        print(solver)
         self.assertAlmostEqual(p3.value, 68.1119420108, places=4)
+        self._check_kkt(p3, places=4)
+
+
+class TestQp(QPTestBase):
+    """Test native QP solvers."""
+
+    def setUp(self) -> None:
+        super().setUp()
+
+        # Check for all installed QP solvers
+        self.solvers = [x for x in QP_SOLVERS if x in INSTALLED_SOLVERS]
+        self.solvers = self.filter_licensed_solvers(self.solvers)
+
+    def solve_QP(self, problem, solver_name):
+        return problem.solve(solver=solver_name, verbose=False)
+
+    def test_all_solvers(self) -> None:
+        for solver in self.solvers:
+            self.quad_over_lin(solver)
+            self.power(solver)
+            self.power_matrix(solver)
+            self.square_affine(solver)
+            self.quad_form(solver)
+            self.affine_problem(solver)
+            self.maximize_problem(solver)
+            self.abs(solver)
+
+            # Do we need the following functionality?
+            # self.norm_2(solver)
+            # self.mat_norm_2(solver)
+
+            self.quad_form_coeff(solver)
+            self.quad_form_bound(solver)
+            self.regression_1(solver)
+            self.regression_2(solver)
+            self.rep_quad_form(solver)
+
+            # slow tests:
+            self.control(solver)
+            self.sparse_system(solver)
+            self.smooth_ridge(solver)
+            self.huber_small(solver)
+            self.huber(solver)
+            self.equivalent_forms_1(solver)
+            self.equivalent_forms_2(solver)
+            if solver != cp.KNITRO:
+                self.equivalent_forms_3(solver)
 
     def test_warm_start(self) -> None:
         """Test warm start.
@@ -504,6 +557,29 @@ class TestQp(BaseTest):
         result2 = prob.solve(solver="OSQP", warm_start=False)
         self.assertAlmostEqual(result, result2)
 
+    def test_qpalm_warmstart(self) -> None:
+        """Test warm start.
+        """
+        if cp.QPALM in INSTALLED_SOLVERS:
+            m = 200
+            n = 100
+            np.random.seed(1)
+            A = np.random.randn(m, n)
+            b = Parameter(m)
+
+            # Construct the problem.
+            x = Variable(n)
+            prob = Problem(Minimize(sum_squares(A @ x - b)))
+
+            b.value = np.random.randn(m)
+            result = prob.solve(solver=cp.QPALM, warm_start=False)
+            result2 = prob.solve(solver=cp.QPALM, warm_start=True)
+            self.assertAlmostEqual(result, result2)
+            b.value = np.random.randn(m)
+            result = prob.solve(solver=cp.QPALM, warm_start=True)
+            result2 = prob.solve(solver=cp.QPALM, warm_start=False)
+            self.assertAlmostEqual(result, result2)
+
     def test_gurobi_warmstart(self) -> None:
         """Test Gurobi warm start with a user provided point.
         """
@@ -606,7 +682,6 @@ class TestQp(BaseTest):
             )
             assert problem.status == cp.OPTIMAL
 
-
     def test_piqp_warmstart(self) -> None:
         """Test warm start.
         """
@@ -629,6 +704,29 @@ class TestQp(BaseTest):
             result = prob.solve(solver=cp.PIQP, warm_start=True)
             result2 = prob.solve(solver=cp.PIQP, warm_start=False)
             self.assertAlmostEqual(result, result2)
+    
+    def test_copt_warmstart(self) -> None:
+        """Test warm start.
+        """
+        if cp.COPT in INSTALLED_SOLVERS:
+            m = 200
+            n = 100
+            np.random.seed(1)
+            A = np.random.randn(m, n)
+            b = Parameter(m)
+            
+            # Construct the problem.
+            x = Variable(n)
+            prob = Problem(Minimize(sum_squares(A @ x - b)))
+
+            b.value = np.random.randn(m)
+            result = prob.solve(solver=cp.COPT, warm_start=False)
+            result2 = prob.solve(solver=cp.COPT, warm_start=True)
+            self.assertAlmostEqual(result, result2)
+            b.value = np.random.randn(m)
+            result = prob.solve(solver=cp.COPT, warm_start=True)
+            result2 = prob.solve(solver=cp.COPT, warm_start=False)
+            self.assertAlmostEqual(result, result2)
 
     def test_parametric(self) -> None:
         """Test solve parametric problem vs full problem"""
@@ -756,6 +854,71 @@ class TestQp(BaseTest):
             self.assertEqual(str(cm.exception), "The solver %s is not installed." % GUROBI)
 
 
+class TestConicQuadObj(QPTestBase):
+    """Test conic solvers with use_quad_obj=True."""
+
+    def setUp(self) -> None:
+        super().setUp()
+
+        # Conic solvers that support quadratic objectives
+        # Exclude KNITRO - its conic interface with use_quad_obj is unstable in CI
+        self.solvers = [
+            solver for solver in INSTALLED_CONIC_SOLVERS
+            if solver in SOLVER_MAP_CONIC
+            and SOLVER_MAP_CONIC[solver].supports_quad_obj()
+            and solver != cp.KNITRO
+        ]
+        self.solvers = self.filter_licensed_solvers(self.solvers)
+
+    def solve_QP(self, problem, solver_name):
+        """Solve with use_quad_obj=True and verify no SOC cones are introduced."""
+        data, _, _ = problem.get_problem_data(
+            solver_name,
+            solver_opts={"use_quad_obj": True}
+        )
+        self.assertEqual(data["dims"].soc, [],
+            f"Problem should have no SOC cones for QP canonicalization with {solver_name}")
+        return problem.solve(solver=solver_name, use_quad_obj=True, verbose=False)
+
+    def test_all_solvers(self) -> None:
+        """Test conic solvers with use_quad_obj=True.
+
+        Only runs tests that have constraints (filtering out unconstrained
+        problems for solvers with REQUIRES_CONSTR=True).
+
+        Tests with m=0 after canonicalization (skip for REQUIRES_CONSTR solvers):
+        - power, quad_form, quad_form_coeff, rep_quad_form
+        """
+        for solver in self.solvers:
+            requires_constr = SOLVER_MAP_CONIC[solver].REQUIRES_CONSTR
+
+            self.quad_over_lin(solver)
+            if not requires_constr:
+                self.power(solver)
+            self.power_matrix(solver)
+            self.square_affine(solver)
+            if not requires_constr:
+                self.quad_form(solver)
+            self.affine_problem(solver)
+            self.maximize_problem(solver)
+            self.abs(solver)
+            if not requires_constr:
+                self.quad_form_coeff(solver)
+            self.quad_form_bound(solver)
+            self.regression_1(solver)
+            self.regression_2(solver)
+            if not requires_constr:
+                self.rep_quad_form(solver)
+            self.control(solver)
+            self.sparse_system(solver)
+            self.smooth_ridge(solver)
+            self.huber_small(solver, places=3)
+            self.huber(solver)
+            self.equivalent_forms_1(solver)
+            self.equivalent_forms_2(solver)
+            self.equivalent_forms_3(solver)
+
+
 @unittest.skipUnless('MPAX' in INSTALLED_SOLVERS, 'MPAX is not installed.')
 class TestMPAX(unittest.TestCase):
 
@@ -801,3 +964,88 @@ class TestMPAX(unittest.TestCase):
 
     def test_MPAX_qp_0(self) -> None:
         StandardTestQPs.test_qp_0(solver='MPAX')
+
+
+class TestQpSolverValidation(unittest.TestCase):
+    """Test QP solver validation of unsupported cone types."""
+
+    def _apply_reductions(self, problem):
+        """Apply the full reduction chain to get a ParamConeProg."""
+        from cvxpy.reductions.cvx_attr2constr import CvxAttr2Constr
+        from cvxpy.reductions.dcp2cone.cone_matrix_stuffing import ConeMatrixStuffing
+        from cvxpy.reductions.dcp2cone.dcp2cone import Dcp2Cone
+        from cvxpy.reductions.flip_objective import FlipObjective
+
+        reductions = []
+        if isinstance(problem.objective, cp.Maximize):
+            reductions.append(FlipObjective())
+        reductions.extend([Dcp2Cone(), CvxAttr2Constr(), ConeMatrixStuffing()])
+
+        reduced = problem
+        for reduction in reductions:
+            reduced = reduction.apply(reduced)[0]
+        return reduced
+
+    def test_qp_solver_rejects_exponential_cones(self) -> None:
+        """Test that QP solver rejects problems with exponential cones."""
+        from cvxpy.error import SolverError
+        from cvxpy.reductions.solvers.qp_solvers.osqp_qpif import OSQP
+
+        # Create a problem with exponential cone (log)
+        x = cp.Variable()
+        prob = cp.Problem(cp.Maximize(cp.log(x)), [x <= 1])
+
+        # Apply reductions to get ParamConeProg
+        param_cone_prog = self._apply_reductions(prob)
+
+        # Test accepts() returns False
+        osqp_solver = OSQP()
+        self.assertFalse(osqp_solver.accepts(param_cone_prog))
+
+        # Test apply() raises SolverError with helpful message
+        with self.assertRaises(SolverError) as cm:
+            osqp_solver.apply(param_cone_prog)
+        self.assertIn("exponential cones", str(cm.exception))
+        self.assertIn("OSQP", str(cm.exception))
+
+    def test_qp_solver_rejects_psd_cones(self) -> None:
+        """Test that QP solver rejects problems with PSD cones."""
+        from cvxpy.error import SolverError
+        from cvxpy.reductions.solvers.qp_solvers.osqp_qpif import OSQP
+
+        # Create a problem with PSD cone
+        X = cp.Variable((2, 2), symmetric=True)
+        prob = cp.Problem(cp.Minimize(cp.trace(X)), [X >> 0, X[0, 0] >= 1])
+
+        # Apply reductions to get ParamConeProg
+        param_cone_prog = self._apply_reductions(prob)
+
+        # Test accepts() returns False
+        osqp_solver = OSQP()
+        self.assertFalse(osqp_solver.accepts(param_cone_prog))
+
+        # Test apply() raises SolverError with helpful message
+        with self.assertRaises(SolverError) as cm:
+            osqp_solver.apply(param_cone_prog)
+        self.assertIn("PSD cones", str(cm.exception))
+
+    def test_qp_solver_rejects_soc_cones(self) -> None:
+        """Test that QP solver rejects problems with second-order cones."""
+        from cvxpy.error import SolverError
+        from cvxpy.reductions.solvers.qp_solvers.osqp_qpif import OSQP
+
+        # Create a problem with SOC (norm)
+        x = cp.Variable(3)
+        prob = cp.Problem(cp.Minimize(cp.norm(x)), [cp.sum(x) == 1])
+
+        # Apply reductions to get ParamConeProg
+        param_cone_prog = self._apply_reductions(prob)
+
+        # Test accepts() returns False
+        osqp_solver = OSQP()
+        self.assertFalse(osqp_solver.accepts(param_cone_prog))
+
+        # Test apply() raises SolverError with helpful message
+        with self.assertRaises(SolverError) as cm:
+            osqp_solver.apply(param_cone_prog)
+        self.assertIn("second-order cones", str(cm.exception))
